@@ -3,32 +3,31 @@ This is the core file in the `gradio` package, and defines the Interface class,
 interface using the input and output types.
 """
 
-import tempfile
-import webbrowser
-
+import gradio
 from gradio.inputs import InputComponent
 from gradio.outputs import OutputComponent
 from gradio import networking, strings, utils
-from distutils.version import StrictVersion
+from gradio.interpretation import quantify_difference_in_label
 import pkg_resources
 import requests
 import random
 import time
+import webbrowser
 import inspect
-from IPython import get_ipython
 import sys
 import weakref
 import analytics
+import numpy as np
 import os
+import copy
+import markdown2
+import json
 
-PKG_VERSION_URL = "https://gradio.app/api/pkg-version"
 analytics.write_key = "uxIFddIEuuUcFLf9VgH2teTEtPlWdkNy"
 analytics_url = 'https://api.gradio.app/'
-try:
-    ip_address = requests.get('https://api.ipify.org').text
-except requests.ConnectionError:
-    ip_address = "No internet connection"
+ip_address = networking.get_local_ip_address()
 
+JSON_PATH = os.path.join(os.path.dirname(gradio.__file__), "launches.json")
 
 class Interface:
     """
@@ -45,11 +44,13 @@ class Interface:
             Interface.instances)
 
     def __init__(self, fn, inputs, outputs, verbose=False, examples=None,
-                 live=False, show_input=True, show_output=True,
-                 capture_session=False, title=None, description=None,
-                 thumbnail=None, server_port=None, server_name=networking.LOCALHOST_NAME,
+                 examples_per_page=10, live=False,
+                 layout="horizontal", show_input=True, show_output=True,
+                 capture_session=False, interpretation=None,
+                 title=None, description=None, article=None, thumbnail=None, 
+                 css=None, server_port=7860, server_name=networking.LOCALHOST_NAME,
                  allow_screenshot=True, allow_flagging=True,
-                 flagging_dir="flagged", analytics_enabled=True):
+                 embedding=None, flagging_dir="flagged", analytics_enabled=True):
 
         """
         Parameters:
@@ -58,11 +59,18 @@ class Interface:
         outputs (Union[str, List[Union[str, OutputComponent]]]): a single Gradio output component, or list of Gradio output components. Components can either be passed as instantiated objects, or referred to by their string shortcuts. The number of output components should match the number of values returned by fn.
         verbose (bool): whether to print detailed information during launch.
         examples (List[List[Any]]): sample inputs for the function; if provided, appears below the UI components and can be used to populate the interface. Should be nested list, in which the outer list consists of samples and each inner list consists of an input corresponding to each input component.
+        examples_per_page (int): If examples are provided, how many to display per page.
         live (bool): whether the interface should automatically reload on change.
+        layout (str): Layout of input and output panels. "horizontal" arranges them as two columns of equal height, "unaligned" arranges them as two columns of unequal height, and "vertical" arranges them vertically.
         capture_session (bool): if True, captures the default graph and session (needed for Tensorflow 1.x)
+        interpretation (Union[Callable, str]): function that provides interpretation explaining prediction output. Pass "default" to use built-in interpreter. 
         title (str): a title for the interface; if provided, appears above the input and output components.
         description (str): a description for the interface; if provided, appears above the input and output components.
+        article (str): an expanded article explaining the interface; if provided, appears below the input and output components. Accepts Markdown and HTML content.
         thumbnail (str): path to image or src to use as display picture for models listed in gradio.app/hub
+        css (str): custom css or path to custom css file to use with interface.
+        server_port (int): will start gradio app on this port (if available) 
+        server_name (str): to make app accessible on local network set to "0.0.0.0".
         allow_screenshot (bool): if False, users will not see a button to take a screenshot of the interface.
         allow_flagging (bool): if False, users will not see a button to flag an input and output.
         flagging_dir (str): what to name the dir where flagged data is stored.
@@ -103,19 +111,31 @@ class Interface:
 
         self.output_interfaces *= len(fn)
         self.predict = fn
+        self.function_names = [func.__name__ for func in fn]
         self.verbose = verbose
         self.status = "OFF"
         self.live = live
+        self.layout = layout
         self.show_input = show_input
         self.show_output = show_output
         self.flag_hash = random.getrandbits(32)
         self.capture_session = capture_session
+        self.interpretation = interpretation            
         self.session = None
         self.server_name = server_name
         self.title = title
         self.description = description
+        if article is not None:
+            article = markdown2.markdown(article)
+        self.article = article
         self.thumbnail = thumbnail
+        if css is not None and os.path.exists(css):
+            with open(css) as css_file:
+                self.css = css_file.read()
+        else:
+            self.css = css
         self.examples = examples
+        self.examples_per_page = examples_per_page
         self.server_port = server_port
         self.simple_server = None
         self.allow_screenshot = allow_screenshot
@@ -123,13 +143,20 @@ class Interface:
         self.flagging_dir = flagging_dir
         Interface.instances.add(self)
         self.analytics_enabled=analytics_enabled
+        self.save_to = None
+        self.share = None
+        self.embedding = embedding
 
         data = {'fn': fn,
                 'inputs': inputs,
                 'outputs': outputs,
                 'live': live,
                 'capture_session': capture_session,
-                'ip_address': ip_address
+                'ip_address': ip_address,
+                'interpretation': interpretation,
+                'embedding': embedding,
+                'allow_flagging': allow_flagging,
+                'allow_screenshot': allow_screenshot,
                 }
 
         if self.capture_session:
@@ -171,13 +198,18 @@ class Interface:
                 for iface in self.output_interfaces],
             "function_count": len(self.predict),
             "live": self.live,
+            "examples_per_page": self.examples_per_page,
+            "layout": self.layout,
             "show_input": self.show_input,
             "show_output": self.show_output,
             "title": self.title,
             "description": self.description,
+            "article": self.article,
             "thumbnail": self.thumbnail,
             "allow_screenshot": self.allow_screenshot,
-            "allow_flagging": self.allow_flagging
+            "allow_flagging": self.allow_flagging,
+            "allow_interpretation": self.interpretation is not None,
+            "allow_embedding": self.embedding is not None,
         }
         try:
             param_names = inspect.getfullargspec(self.predict[0])[0]
@@ -185,48 +217,42 @@ class Interface:
                 if not iface[1]["label"]:
                     iface[1]["label"] = param.replace("_", " ")
             for i, iface in enumerate(config["output_interfaces"]):
-                ret_name = "Output " + str(i + 1) if len(config["output_interfaces"]) > 1 else "Output"
+                outputs_per_function = int(len(self.output_interfaces) / len(self.predict))
+                function_index = i // outputs_per_function
+                component_index = i - function_index * outputs_per_function
+                ret_name = "Output " + str(component_index + 1) if outputs_per_function > 1 else "Output"
                 if not iface[1]["label"]:
                     iface[1]["label"] = ret_name
+                if len(self.predict) > 1:
+                    iface[1]["label"] = self.function_names[function_index].replace("_", " ") + ": " + iface[1]["label"]
+                    
         except ValueError:
             pass
-
+        if self.examples is not None:
+            processed_examples = []
+            for example_set in self.examples:
+                processed_set = []
+                for iface, example in zip(self.input_interfaces, example_set):
+                    processed_set.append(example)
+                processed_examples.append(processed_set)
+            config["examples"] = processed_examples
         return config
 
-    def process(self, raw_input, predict_fn=None):
-        """
-        :param raw_input: a list of raw inputs to process and apply the
-        prediction(s) on.
-        :param predict_fn: which function to process. If not provided, all of the model functions are used.
-        :return:
-        processed output: a list of processed  outputs to return as the
-        prediction(s).
-        duration: a list of time deltas measuring inference time for each
-        prediction fn.
-        """
-        processed_input = [input_interface.preprocess(raw_input[i])
-                           for i, input_interface in enumerate(self.input_interfaces)]
+    def run_prediction(self, processed_input, return_duration=False):
         predictions = []
         durations = []
         for predict_fn in self.predict:
             start = time.time()
-            if self.capture_session and not (self.session is None):
+            if self.capture_session and self.session is not None:
                 graph, sess = self.session
-                with graph.as_default():
-                    with sess.as_default():
-                        prediction = predict_fn(*processed_input)
+                with graph.as_default(), sess.as_default():
+                    prediction = predict_fn(*processed_input)
             else:
                 try:
                     prediction = predict_fn(*processed_input)
                 except ValueError as exception:
-                    if str(exception).endswith("is not an element of this "
-                                               "graph."):
-                        raise ValueError("It looks like you might be using "
-                                         "tensorflow < 2.0. Please "
-                                         "pass capture_session=True in "
-                                         "Interface to avoid the 'Tensor is "
-                                         "not an element of this graph.' "
-                                         "error.")
+                    if str(exception).endswith("is not an element of this graph."):
+                        raise ValueError(strings.en["TF1_ERROR"])
                     else:
                         raise exception
             duration = time.time() - start
@@ -235,9 +261,87 @@ class Interface:
                 prediction = [prediction]
             durations.append(duration)
             predictions.extend(prediction)
+        
+        if return_duration:
+            return predictions, durations
+        else:
+            return predictions
+
+    def process(self, raw_input):
+        """
+        :param raw_input: a list of raw inputs to process and apply the prediction(s) on.
+        processed output: a list of processed  outputs to return as the prediction(s).
+        duration: a list of time deltas measuring inference time for each prediction fn.
+        """
+        processed_input = [input_interface.preprocess(raw_input[i])
+                           for i, input_interface in enumerate(self.input_interfaces)]
+        predictions, durations = self.run_prediction(processed_input, return_duration=True)
         processed_output = [output_interface.postprocess(
             predictions[i]) for i, output_interface in enumerate(self.output_interfaces)]
         return processed_output, durations
+    
+    def embed(self, processed_input):
+        if self.embedding == "default":
+            embeddings = np.concatenate([input_interface.embed(processed_input[i])
+                            for i, input_interface in enumerate(self.input_interfaces)])
+        else:
+            embeddings = self.embedding(*processed_input)
+        return embeddings
+
+    def interpret(self, raw_input):
+        """
+        Runs the interpretation command for the machine learning model. Handles both the "default" out-of-the-box
+        interpretation for a certain set of UI component types, as well as the custom interpretation case.
+        :param raw_input: a list of raw inputs to apply the interpretation(s) on.
+        """
+        if self.interpretation == "default":
+            processed_input = [input_interface.preprocess(raw_input[i])
+                            for i, input_interface in enumerate(self.input_interfaces)]
+            original_output = self.run_prediction(processed_input)
+            scores, alternative_outputs = [], []
+            for i, x in enumerate(raw_input):
+                input_interface = self.input_interfaces[i]
+                neighbor_raw_input = list(raw_input)
+                neighbor_values, interpret_kwargs, interpret_by_removal = input_interface.get_interpretation_neighbors(x)
+                interface_scores = []
+                alternative_output = []
+                for neighbor_input in neighbor_values:
+                    neighbor_raw_input[i] = neighbor_input
+                    processed_neighbor_input = [input_interface.preprocess(neighbor_raw_input[i])
+                                    for i, input_interface in enumerate(self.input_interfaces)]
+                    neighbor_output = self.run_prediction(processed_neighbor_input)
+                    processed_neighbor_output = [output_interface.postprocess(
+                        neighbor_output[i]) for i, output_interface in enumerate(self.output_interfaces)]
+
+                    alternative_output.append(processed_neighbor_output)
+                    interface_scores.append(quantify_difference_in_label(self, original_output, neighbor_output))
+                alternative_outputs.append(alternative_output)
+                if not interpret_by_removal:
+                    interface_scores = [-score for score in interface_scores]
+                scores.append(
+                    input_interface.get_interpretation_scores(
+                        raw_input[i], neighbor_values, interface_scores, **interpret_kwargs))
+            return scores, alternative_outputs
+        else:
+            processed_input = [input_interface.preprocess(raw_input[i])
+                               for i, input_interface in enumerate(self.input_interfaces)]
+            interpreter = self.interpretation
+
+            if self.capture_session and self.session is not None:
+                graph, sess = self.session
+                with graph.as_default(), sess.as_default():
+                    interpretation = interpreter(*processed_input)
+            else:
+                try:
+                    interpretation = interpreter(*processed_input)
+                except ValueError as exception:
+                    if str(exception).endswith("is not an element of this graph."):
+                        raise ValueError(strings.en["TF1_ERROR"])
+                    else:
+                        raise exception
+            if len(raw_input) == 1:
+                interpretation = [interpretation]
+            return interpretation, []
 
     def close(self):
         if self.simple_server and not (self.simple_server.fileno() == -1):  # checks to see if server is running
@@ -251,11 +355,11 @@ class Interface:
         except (KeyboardInterrupt, OSError):
             print("Keyboard interruption in main thread... closing server.")
             thread.keep_running = False
-            networking.url_ok(path_to_local_server)
+            networking.url_ok(path_to_local_server)  # Hit the server one more time to close it
 
     def test_launch(self):
         for predict_fn in self.predict:
-            print("Test launching: {}()...".format(predict_fn.__name__), end=' ')
+            print("Test launch: {}()...".format(predict_fn.__name__), end=' ')
 
             raw_input = []
             for input_interface in self.input_interfaces:
@@ -269,102 +373,77 @@ class Interface:
                 print("PASSED")
                 continue
 
-    def launch(self, inline=None, inbrowser=None, share=False, debug=False):
+    def launch(self, inline=None, inbrowser=None, share=False, debug=False, auth=None):
         """
-        Parameters
+        Parameters:
         inline (bool): whether to display in the interface inline on python notebooks.
         inbrowser (bool): whether to automatically launch the interface in a new tab on the default browser.
         share (bool): whether to create a publicly shareable link from your computer for the interface.
         debug (bool): if True, and the interface was launched from Google Colab, prints the errors in the cell output.
-        Returns
-        httpd (str): HTTPServer object
+        auth (Tuple[str, str]): If provided, username and password required to access interface.
+        Returns:
+        app (flask.Flask): Flask app object
         path_to_local_server (str): Locally accessible link
         share_url (str): Publicly accessible link (if share=True)
         """
-        output_directory = tempfile.mkdtemp()
-        # Set up a port to serve the directory containing the static files with interface.
-        server_port, httpd, thread = networking.start_simple_server(
-            self, output_directory, self.server_name, server_port=self.server_port)
-        path_to_local_server = "http://{}:{}/".format(self.server_name, server_port)
-        networking.build_template(output_directory)
+        # Alert user if a more recent version of the library exists
+        utils.version_check()
 
+        # Set up local flask server
+        config = self.get_config_file()
+        networking.set_config(config)
+        networking.set_meta_tags(self.title, self.description, self.thumbnail)
+        self.auth = auth
+
+        # Launch local flask server
+        server_port, app, thread = networking.start_server(
+            self, self.server_name, self.server_port, self.auth)
+        path_to_local_server = "http://{}:{}/".format(self.server_name, server_port)
         self.server_port = server_port
         self.status = "RUNNING"
-        self.simple_server = httpd
+        self.server = app
 
-        try:
-            current_pkg_version = pkg_resources.require("gradio")[0].version
-            latest_pkg_version = requests.get(url=PKG_VERSION_URL).json()["version"]
-            if StrictVersion(latest_pkg_version) > StrictVersion(current_pkg_version):
-                print("IMPORTANT: You are using gradio version {}, "
-                      "however version {} "
-                      "is available, please upgrade.".format(
-                    current_pkg_version, latest_pkg_version))
-                print('--------')
-        except:  # TODO(abidlabs): don't catch all exceptions
-            pass
+        # Count number of launches
+        launch_counter()
 
+        # If running in a colab or not able to access localhost, automatically create a shareable link        
         is_colab = utils.colab_check()
-        if not is_colab:
-            if not networking.url_ok(path_to_local_server):
-                share = True
-            else:
-                print(strings.en["RUNNING_LOCALLY"].format(path_to_local_server))
-        else:
+        if is_colab or not(networking.url_ok(path_to_local_server)):  
+            share = True
             if debug:
-                print("Colab notebook detected. This cell will run indefinitely so that you can see errors and logs. "
-                      "To turn off, set debug=False in launch().")
+                print(strings.en["COLAB_DEBUG_TRUE"])
             else:
-                print("Colab notebook detected. To show errors in colab notebook, set debug=True in launch()")
+                print(strings.en["COLAB_DEBUG_FALSE"])
+        else:
+            print(strings.en["RUNNING_LOCALLY"].format(path_to_local_server))
 
+        # Set up shareable link 
+        self.share = share
         if share:
-            print("This share link will expire in 6 hours. If you need a "
-                  "permanent link, email support@gradio.app")
+            print(strings.en["SHARE_LINK_MESSAGE"])
             try:
                 share_url = networking.setup_tunnel(server_port)
-                print("Running on External URL:", share_url)
+                print(strings.en["SHARE_LINK_DISPLAY"].format(share_url))
             except RuntimeError:
-                data = {'error': 'RuntimeError in launch method'}
-                if self.analytics_enabled:
-                    try:
-                        requests.post(analytics_url + 'gradio-error-analytics/',
-                                      data=data)
-                    except requests.ConnectionError:
-                        pass  # do not push analytics if no network
+                send_error_analytics(self.analytics_enabled)
                 share_url = None
-                if self.verbose:
-                    print(strings.en["NGROK_NO_INTERNET"])
         else:
-            if is_colab:  # For a colab notebook, create a public link even if
-                # share is False.
-                share_url = networking.setup_tunnel(server_port)
-                print("Running on External URL:", share_url)
-                if self.verbose:
-                    print(strings.en["COLAB_NO_LOCAL"])
-            else:  # If it's not a colab notebook and share=False, print a message telling them about the share option.
-                print("To get a public link for a hosted model, "
-                      "set Share=True")
-                if self.verbose:
-                    print(strings.en["PUBLIC_SHARE_TRUE"])
-                share_url = None
+            print(strings.en["PUBLIC_SHARE_TRUE"])
+            share_url = None
 
+        # Open a browser tab with the interface.
+        if inbrowser: 
+            if share:
+                webbrowser.open(share_url) 
+            else:
+                webbrowser.open(path_to_local_server) 
+                
+        # Check if running in a Python notebook in which case, display inline
         if inline is None:
             inline = utils.ipython_check()
-            if inbrowser is None:
-                # if interface won't appear inline, open it in new tab,
-                # otherwise keep it inline
-                inbrowser = not inline
-        else:
-            if inbrowser is None:
-                inbrowser = False
-
-        if inbrowser and not is_colab:
-            webbrowser.open(path_to_local_server)  # Open a browser tab
-            # with the interface.
         if inline:
             from IPython.display import IFrame, display
-            # Embed the remote interface page if on google colab;
-            # otherwise, embed the local page.
+            # Embed the remote interface page if on google colab; otherwise, embed the local page.
             print("Interface loading below...")
             if share:
                 while not networking.url_ok(share_url):
@@ -373,46 +452,61 @@ class Interface:
             else:
                 display(IFrame(path_to_local_server, width=1000, height=500))
 
-        config = self.get_config_file()
-        config["share_url"] = share_url
+        send_launch_analytics(analytics_enabled=self.analytics_enabled, inbrowser=inbrowser, is_colab=is_colab, 
+                              share=share, share_url=share_url)
 
-        processed_examples = []
-        if self.examples is not None:
-            for example_set in self.examples:
-                processed_set = []
-                for iface, example in zip(self.input_interfaces, example_set):
-                    processed_set.append(iface.process_example(example))
-                processed_examples.append(processed_set)
-            config["examples"] = processed_examples
-
-        networking.set_config(config, output_directory)
-        networking.set_meta_tags(output_directory, self.title, self.description, self.thumbnail)
-
-        if debug:
+        # Run server perpetually under certain circumstances
+        if debug or int(os.getenv('GRADIO_DEBUG', 0))==1:
             while True:
                 sys.stdout.flush()
                 time.sleep(0.1)
-
-        launch_method = 'browser' if inbrowser else 'inline'
-        data = {'launch_method': launch_method,
-                'is_google_colab': is_colab,
-                'is_sharing_on': share,
-                'share_url': share_url,
-                'ip_address': ip_address
-                }
-
-        if self.analytics_enabled:
-            try:
-                requests.post(analytics_url + 'gradio-launched-analytics/',
-                              data=data)
-            except requests.ConnectionError:
-                pass  # do not push analytics if no network
-
         is_in_interactive_mode = bool(getattr(sys, 'ps1', sys.flags.interactive))
         if not is_in_interactive_mode:
             self.run_until_interrupted(thread, path_to_local_server)
+        
+        return app, path_to_local_server, share_url
 
-        return httpd, path_to_local_server, share_url
+def launch_counter():
+    try:
+        if not os.path.exists(JSON_PATH):
+            launches = {"launches": 1}
+            with open(JSON_PATH, "w+") as j:
+                json.dump(launches, j)
+        else:
+            with open(JSON_PATH) as j:
+                launches = json.load(j)
+            launches["launches"] += 1
+            if launches["launches"] in [25, 50]:
+                print(strings.en["BETA_INVITE"])
+            with open(JSON_PATH, "w") as j:
+                j.write(json.dumps(launches))
+    except:
+        pass
+
+def send_error_analytics(analytics_enabled):
+    data = {'error': 'RuntimeError in launch method'}
+    if analytics_enabled:
+        try:
+            requests.post(analytics_url + 'gradio-error-analytics/',
+                            data=data)
+        except requests.ConnectionError:
+            pass  # do not push analytics if no network
+
+def send_launch_analytics(analytics_enabled, inbrowser, is_colab, share, share_url):
+    launch_method = 'browser' if inbrowser else 'inline'
+    if analytics_enabled:
+        data = {
+            'launch_method': launch_method,
+            'is_google_colab': is_colab,
+            'is_sharing_on': share,
+            'share_url': share_url,
+            'ip_address': ip_address
+        }
+        try:
+            requests.post(analytics_url + 'gradio-launched-analytics/',
+                            data=data)
+        except requests.ConnectionError:
+            pass  # do not push analytics if no network
 
 
 def reset_all():
