@@ -15,12 +15,22 @@ import warnings
 import tempfile
 import scipy
 import os
+import pandas as pd
+import PIL
+from types import ModuleType
 
 class OutputComponent(Component):
     """
     Output Component. All output components subclass this.
     """
-    pass
+
+    def postprocess(self, y):
+        """
+        Any postprocessing needed to be performed on function output.
+        """
+        return y
+
+
 
 class Textbox(OutputComponent):
     '''
@@ -28,10 +38,10 @@ class Textbox(OutputComponent):
     Output type: Union[str, float, int]
     '''
 
-    def __init__(self, type="str", label=None):
+    def __init__(self, type="auto", label=None):
         '''
         Parameters:
-        type (str): Type of value to be passed to component. "str" expects a string, "number" expects a float value.
+        type (str): Type of value to be passed to component. "str" expects a string, "number" expects a float value, "auto" detects return type.
         label (str): component name in interface.
         '''
         self.type = type
@@ -51,10 +61,10 @@ class Textbox(OutputComponent):
         }
 
     def postprocess(self, y):
-        if self.type == "str":
-            return y
-        elif self.type == "number":
+        if self.type == "str" or self.type == "auto":
             return str(y)
+        elif self.type == "number":
+            return y
         else:
             raise ValueError("Unknown type: " + self.type + ". Please choose from: 'str', 'number'")
 
@@ -65,23 +75,23 @@ class Label(OutputComponent):
     Output type: Union[Dict[str, float], str, int, float]
     '''
 
-    LABEL_KEY = "label"
-    CONFIDENCE_KEY = "confidence"
     CONFIDENCES_KEY = "confidences"
 
-    def __init__(self, num_top_classes=None, label=None):
+    def __init__(self, num_top_classes=None, type="auto", label=None):
         '''
         Parameters:
         num_top_classes (int): number of most confident classes to show.
+        type (str): Type of value to be passed to component. "value" expects a single out label, "confidences" expects a dictionary mapping labels to confidence scores, "auto" detects return type.
         label (str): component name in interface.
         '''
         self.num_top_classes = num_top_classes
+        self.type = type
         super().__init__(label)
 
     def postprocess(self, y):
-        if isinstance(y, str) or isinstance(y, Number):
+        if self.type == "label" or (self.type == "auto" and (isinstance(y, str) or isinstance(y, Number))):
             return {"label": str(y)}
-        elif isinstance(y, dict):
+        elif self.type == "confidences" or (self.type == "auto" and isinstance(y, dict)):
             sorted_pred = sorted(
                 y.items(),
                 key=operator.itemgetter(1),
@@ -90,16 +100,14 @@ class Label(OutputComponent):
             if self.num_top_classes is not None:
                 sorted_pred = sorted_pred[:self.num_top_classes]
             return {
-                self.LABEL_KEY: sorted_pred[0][0],
-                self.CONFIDENCES_KEY: [
+                "label": sorted_pred[0][0],
+                "confidences": [
                     {
-                        self.LABEL_KEY: pred[0],
-                        self.CONFIDENCE_KEY: pred[1]
+                        "label": pred[0],
+                        "confidence": pred[1]
                     } for pred in sorted_pred
                 ]
             }
-        elif isinstance(y, int) or isinstance(y, float):
-            return {self.LABEL_KEY: str(y)}
         else:
             raise ValueError("The `Label` output interface expects one of: a string label, or an int label, a "
                              "float label, or a dictionary whose keys are labels and values are confidences.")
@@ -120,17 +128,18 @@ class Label(OutputComponent):
 class Image(OutputComponent):
     '''
     Component displays an output image. 
-    Output type: Union[numpy.array, PIL.Image, str, matplotlib.pyplot]
+    Output type: Union[numpy.array, PIL.Image, str, matplotlib.pyplot, Tuple[Union[numpy.array, PIL.Image, str], List[Tuple[str, float, float, float, float]]]]
     '''
 
-    def __init__(self, type="numpy", plot=False, label=None):
+    def __init__(self, type="auto", labeled_segments=False, plot=False, label=None):
         '''
         Parameters:
-        type (str): Type of value to be passed to component. "numpy" expects a numpy array with shape (width, height, 3), "pil" expects a PIL image object, "file" expects a file path to the saved image, "plot" expects a matplotlib.pyplot object.
-        plot (bool): DEPRECATED. Whether to expect a plot figure to be
-        returned by the function.
+        type (str): Type of value to be passed to component. "numpy" expects a numpy array with shape (width, height, 3), "pil" expects a PIL image object, "file" expects a file path to the saved image, "plot" expects a matplotlib.pyplot object, "auto" detects return type.
+        labeled_segments (bool): If True, expects a two-element tuple to be returned. The first element of the tuple is the image of format specified by type. The second element is a list of tuples, where each tuple represents a labeled segment within the image. The first element of the tuple is the string label of the segment, followed by 4 floats that represent the left-x, top-y, right-x, and bottom-y coordinates of the bounding box.
+        plot (bool): DEPRECATED. Whether to expect a plot to be returned by the function.
         label (str): component name in interface.
         '''
+        self.labeled_segments = labeled_segments
         if plot:
             warnings.warn("The 'plot' parameter has been deprecated. Set parameter 'type' to 'plot' instead.", DeprecationWarning)
             self.type = "plot"
@@ -142,21 +151,40 @@ class Image(OutputComponent):
     def get_shortcut_implementations(cls):
         return {
             "image": {},
+            "segmented_image": {"labeled_segments": True},
             "plot": {"type": "plot"},
             "pil": {"type": "pil"}
         }
 
     def postprocess(self, y):
-        if self.type in ["numpy", "pil"]:
-            if self.type == "pil":
-                y = np.array(y)
-            return processing_utils.encode_array_to_base64(y)
-        elif self.type == "file":
-            return processing_utils.encode_file_to_base64(y)
-        elif self.type == "plot":
-            return processing_utils.encode_plot_to_base64(y)
+        if self.labeled_segments:
+            y, coordinates = y
         else:
-            raise ValueError("Unknown type: " + self.type + ". Please choose from: 'numpy', 'pil', 'file', 'plot'.")
+            coordinates = []
+        if self.type == "auto":
+            if isinstance(y, np.ndarray):
+                dtype = "numpy"
+            elif isinstance(y, PIL.Image.Image):
+                dtype = "pil"
+            elif isinstance(y, str):
+                dtype = "file"
+            elif isinstance(y, ModuleType):
+                dtype = "plot"
+            else:
+                raise ValueError("Unknown type. Please choose from: 'numpy', 'pil', 'file', 'plot'.")
+        else:
+            dtype = self.type
+        if dtype in ["numpy", "pil"]:
+            if dtype == "pil":
+                y = np.array(y)
+            out_y = processing_utils.encode_array_to_base64(y)
+        elif dtype == "file":
+            out_y = processing_utils.encode_file_to_base64(y)
+        elif dtype == "plot":
+            out_y = processing_utils.encode_plot_to_base64(y)
+        else:
+            raise ValueError("Unknown type: " + dtype + ". Please choose from: 'numpy', 'pil', 'file', 'plot'.")
+        return out_y, coordinates
 
     def rebuild(self, dir, data):
         """
@@ -164,8 +192,7 @@ class Image(OutputComponent):
         """
         im = processing_utils.decode_base64_to_image(data)
         timestamp = datetime.datetime.now()
-        filename = 'output_{}.png'.format(timestamp.
-                                          strftime("%Y-%m-%d-%H-%M-%S"))
+        filename = 'output_{}_{}.png'.format(self.label, timestamp.strftime("%Y-%m-%d-%H-%M-%S"))
         im.save('{}/{}'.format(dir, filename), 'PNG')
         return filename
 
@@ -236,10 +263,10 @@ class Audio(OutputComponent):
     Output type: Union[Tuple[int, numpy.array], str]
     '''
 
-    def __init__(self, type="numpy", label=None):
+    def __init__(self, type="auto", label=None):
         '''
         Parameters:
-        type (str): Type of value to be passed to component. "numpy" returns a 2-set tuple with an integer sample_rate and the data numpy.array of shape (samples, 2), "file" returns a temporary file path to the saved wav audio file.
+        type (str): Type of value to be passed to component. "numpy" returns a 2-set tuple with an integer sample_rate and the data numpy.array of shape (samples, 2), "file" returns a temporary file path to the saved wav audio file, "auto" detects return type.
         label (str): component name in interface.
         '''
         self.type = type
@@ -257,9 +284,9 @@ class Audio(OutputComponent):
         }
 
     def postprocess(self, y):
-        if self.type in ["numpy", "file"]:
-            if self.type == "numpy":
-                file = tempfile.NamedTemporaryFile()
+        if self.type in ["numpy", "file", "auto"]:
+            if self.type == "numpy" or (self.type == "auto" and isinstance(y, tuple)):
+                file = tempfile.NamedTemporaryFile(delete=False)
                 scipy.io.wavfile.write(file, y[0], y[1])                
                 y = file.name
             return processing_utils.encode_file_to_base64(y, type="audio", ext="wav")
@@ -349,11 +376,11 @@ class Dataframe(OutputComponent):
     Output type: Union[pandas.DataFrame, numpy.array, List[Union[str, float]], List[List[Union[str, float]]]]
     """
 
-    def __init__(self, headers=None, type="pandas", label=None):
+    def __init__(self, headers=None, type="auto", label=None):
         '''
         Parameters:
         headers (List[str]): Header names to dataframe.
-        type (str): Type of value to be passed to component. "pandas" for pandas dataframe, "numpy" for numpy array, or "array" for Python array.
+        type (str): Type of value to be passed to component. "pandas" for pandas dataframe, "numpy" for numpy array, or "array" for Python array, "auto" detects return type.
         label (str): component name in interface.
         '''
         self.type = type
@@ -370,17 +397,26 @@ class Dataframe(OutputComponent):
     @classmethod
     def get_shortcut_implementations(cls):
         return {
-            "dataframe": {"type": "pandas"},
+            "dataframe": {},
             "numpy": {"type": "numpy"},
             "matrix": {"type": "array"},
             "list": {"type": "array"},
         }
 
     def postprocess(self, y):
-        if self.type == "pandas":
+        if self.type == "auto":
+            if isinstance(y, pd.core.frame.DataFrame):
+                dtype = "pandas"
+            elif isinstance(y, np.ndarray):
+                dtype = "numpy"
+            elif isinstance(y, list):
+                dtype = "array"
+        else:
+            dtype = self.type
+        if dtype == "pandas":
             return {"headers": list(y.columns), "data": y.values.tolist()}
-        elif self.type in ("numpy", "array"):
-            if self.type == "numpy":
+        elif dtype in ("numpy", "array"):
+            if dtype == "numpy":
                 y = y.tolist()
             if len(y) == 0 or not isinstance(y[0], list):
                 y = [y]
